#include <_Custom_Charts/Enums.mqh>
#include <_Custom_Charts/Indicadores/Indicadores.mqh>

class MovingAverage : public Indicadores {
private:
    double bufferMedia[];
    double bufferDesvioSuperior[];
    double bufferDesvioInferior[];

public:
    MovingAverage(){};
    ~MovingAverage(){};

    //Definição padrão do indicador
    virtual void set();
    //Definição padrão do indicador
    void set(int periodo_, double desvio_, ENUM_MA_TYPE tipo_, int espelho_);
    //Atualização dos buffers copiando um certo número de posições
    virtual void updateBuffers(int qtdPosCopiadas);
    //Atualização dos buffers
    virtual void updateBuffers();
    //Retorno de um valor do buffer em uma posição
    virtual double getBufferValue(int bufferEnum, int bufferPos);
    //O valor do buffer no candle atual
    virtual double getBufferLast(int bufferEnum);
};

void MovingAverage::set(){
    this.set(60, 20, CLOSE, 1);
}

void MovingAverage::set(int periodo_, double desvio_, ENUM_MA_TYPE tipo_, int espelho_){
    if(periodo_ > 0){
        this.setManipulador(iCustom(_Symbol, _Period, "MovingAverageNTV", periodo_, desvio_, tipo_, espelho_));
    }else
        this.set();
}

void MovingAverage::updateBuffers(int qtdPosCopiadas){
    int manipuladorLocal = this.getManipulador();
    if(manipuladorLocal != INVALID_HANDLE){
        CopyBuffer(manipuladorLocal, 0, 0, qtdPosCopiadas, bufferMedia);
        CopyBuffer(manipuladorLocal, 1, 0, qtdPosCopiadas, bufferDesvioSuperior);
        CopyBuffer(manipuladorLocal, 2, 0, qtdPosCopiadas, bufferDesvioInferior);
    }
}

void MovingAverage::updateBuffers(){
    updateBuffers(1);
}

double MovingAverage::getBufferValue(int bufferEnum, int bufferPos){
    //Faz cast do valor inserido da relacao do buffer
    ENUM_MA_BUFFERS bufferRetorno = (ENUM_MA_BUFFERS) bufferEnum;
    //Atualiza os buffers
    this.updateBuffers(bufferPos);
    //Define o index de acesso no buffer e verifica se é valido
    int bufferAcess = bufferPos-1;
    if(bufferAcess > ArraySize(bufferMedia) || bufferAcess < 0)
        return 0;
    switch (bufferRetorno){
    case MA_BUFFER:
        return this.bufferMedia[0];
        break;
    case MA_UP_DEVIATION:
        return this.bufferDesvioSuperior[0];
        break;
    case MA_DOWN_DEVIATION:
        return this.bufferDesvioInferior[0];
        break;
    default:
        return 0;
        break;
    }
}

double MovingAverage::getBufferLast(int bufferEnum){
    return getBufferValue(bufferEnum, 1);
}